@@ -1,3 +1,5 @@
+# coding: iso-8859-15
+
 import os
 import glob
 import shutil
@@ -10,10 +12,11 @@ import math
 import datetime as dt
 import numpy as np
 import ephem
+import pytimeparse
 from astropy import wcs
 from astropy.io import fits
 from astropy.io import votable
-from astropy.time import Time
+from astropy.time import Time, TimeDelta
 from astropy.coordinates import Angle
 from astropy import units
 from collections import OrderedDict
@@ -54,6 +57,7 @@ _keyword_meta = OrderedDict([
     ('fits_maxval', (float, False, None, 'MAXVAL', None)),
     ('fits_extend', (bool, False, True, 'EXTEND', None)),
     ('date_orig', (str, True, [], 'DATEORIG', 'DATEORn')),
+    ('date_orig_end', (str, True, [], None, None)),
     ('tms_orig', (str, True, [], 'TMS-ORIG', 'TMS-ORn')),
     ('tme_orig', (str, True, [], 'TME-ORIG', 'TME-ORn')),
     ('tz_orig', (str, False, None, None, None)),
@@ -71,6 +75,7 @@ _keyword_meta = OrderedDict([
     ('object_type', (str, True, [], 'OBJTYPE', 'OBJTYPn')),
     ('numexp', (int, False, 1, 'NUMEXP', None)),
     ('exptime', (float, True, [], 'EXPTIME', 'EXPTIMn')),
+    ('numsub', (int, True, [], None, None)),
     ('observatory', (str, False, None, 'OBSERVAT', None)),
     ('site_name', (str, False, None, 'SITENAME', None)),
     ('site_latitude', (float, False, None, 'SITELAT', None)),
@@ -659,8 +664,8 @@ class ArchiveMeta:
                 filename = os.path.join(self.fits_dir, filename)
 
             try:
-                header = fits.getheader(filename)
-            except:
+                header = fits.getheader(filename, ignore_missing_end=True)
+            except Exception:
                 print 'Error reading {}'.format(filename)
                 header = None
 
@@ -768,6 +773,7 @@ class LogpageMeta(OrderedDict):
             self[k] = copy.deepcopy(v[1])
 
         self.logpage_dir = ''
+        self.cover_dir = ''
         self['filename'] = filename
         fmt = filename.split('.')[-1].upper()
 
@@ -790,10 +796,11 @@ class LogpageMeta(OrderedDict):
 
         self.conf = conf
 
-        try:
-            setattr(self, 'logpage_dir', conf.get('Files', 'logpage_dir'))
-        except ConfigParser.Error:
-            pass
+        for attr in ['logpage_dir', 'cover_dir']:
+            try:
+                setattr(self, attr, conf.get('Files', attr))
+            except ConfigParser.Error:
+                pass
 
     def parse_csv(self, val_list, csv_filename=None):
         """
@@ -831,6 +838,12 @@ class LogpageMeta(OrderedDict):
 
         fn_path = os.path.join(self.logpage_dir, self['filename'])
 
+        if not os.path.exists(fn_path):
+            fn_path = os.path.join(self.cover_dir, self['filename'])
+
+            if not os.path.exists(fn_path):
+                return
+
         if gexiv_available:
             try:
                 exif = GExiv2.Metadata(fn_path)
@@ -842,10 +855,14 @@ class LogpageMeta(OrderedDict):
 
             try:
                 self['image_datetime'] = exif.get_date_time().isoformat()
-            except KeyError:
+            except (KeyError, TypeError):
                 pass
         else:
-            im_pil = Image.open(fn_path)
+            try:
+                im_pil = Image.open(fn_path)
+            except Exception:
+                return
+            
             self['image_width'], self['image_height'] = im_pil.size
             self['file_format'] = im_pil.format
             exif_datetime = None
@@ -855,12 +872,12 @@ class LogpageMeta(OrderedDict):
 
                 try:
                     exif_datetime = exif[306]
-                except KeyError:
+                except (KeyError, TypeError):
                     pass
             elif self['file_format'] == 'TIFF':
                 try:
                     exif_datetime = im_pil.tag[306]
-                except KeyError:
+                except (KeyError, TypeError):
                     pass
 
             if exif_datetime:
@@ -894,6 +911,7 @@ class PlateMeta(OrderedDict):
             self[k] = copy.deepcopy(v[2])
 
         self['plate_id'] = plate_id
+        self.exposures = None
 
         self.conf = None
         self.output_db_host = 'localhost'
@@ -901,6 +919,14 @@ class PlateMeta(OrderedDict):
         self.output_db_name = ''
         self.output_db_passwd = ''
 
+    def copy(self):
+        pmeta = PlateMeta()
+
+        for k,v in self.items():
+            pmeta[k] = copy.deepcopy(v)
+
+        return pmeta
+
     _object_types = {'A1': 'planet',
         'A2': 'Moon',
         'A3': 'Sun',
@@ -997,8 +1023,10 @@ class PlateMeta(OrderedDict):
             else:
                 if len(self[key]) > exp:
                     val = self[key][exp]
-                else:
+                elif len(self[key]) == 1:
                     val = self[key][0]
+                else:
+                    val = None
         else:
             val = self[key]
 
@@ -1262,7 +1290,10 @@ class PlateMeta(OrderedDict):
             key, val = [s.strip() for s in keyval.split(':', 1)]
 
             if key.lower() == 'temperature':
-                self['temperature'] = float(val.strip())
+                try:
+                    self['temperature'] = float(val.strip())
+                except ValueError:
+                    self['temperature'] = None
             elif key.lower() == 'calmness':
                 self['calmness'] = val.strip()
             elif key.lower() == 'sharpness':
@@ -1321,6 +1352,9 @@ class PlateMeta(OrderedDict):
                             else:
                                 val = [x.strip() for x in 
                                        val_list[pos-1].split(csv_list_delimiter)]
+                                
+                                if val == ['']:
+                                    val = []
                         elif (_keyword_meta[key][0] is int or 
                               _keyword_meta[key][0] is float):
                             val = str_to_num(val_list[pos-1])
@@ -1330,6 +1364,10 @@ class PlateMeta(OrderedDict):
                         if key == 'numexp' and not val:
                             val = 1
 
+                        # Delete previous values for lists
+                        if isinstance(self[key], list):
+                            self[key] = []
+
                         try:
                             self[key].extend(val)
                         except TypeError:
@@ -1352,12 +1390,22 @@ class PlateMeta(OrderedDict):
                     nkey = v[4].replace('n', str(i+1))
 
                     if nkey in header:
-                        self[k].append(header[nkey])
+                        val = header[nkey]
+
+                        if isinstance(val, fits.card.Undefined):
+                            val = None
+
+                        self[k].append(val)
             elif k == 'fits_history':
                 if 'HISTORY' in header:
                     self[k].extend(header['HISTORY'])
             elif v[3] and v[3] in header:
-                self[k] = header[v[3]]
+                val = header[v[3]]
+
+                if isinstance(val, fits.card.Undefined):
+                    val = None
+
+                self[k] = val
 
         if ('fits_bzero' in self and 'fits_bitpix' in self and 
             self['fits_bzero'] and self['fits_bitpix']):
@@ -1412,16 +1460,53 @@ class PlateMeta(OrderedDict):
         """
 
         if self['date_orig']:
-            ntimes = len(self['tms_orig'])
+            tms_missing = False
 
-            if ntimes == 0:
+            if isinstance(self['tms_orig'], list):
+                ntimes = len(self['tms_orig'])
+
+                if ntimes == 0:
+                    tms_missing = True
+                    ntimes = 1
+            else:
                 ntimes = 1
+
+            # Update number of exposures if the number of timestamps is 
+            # different
+            if (ntimes > self['numexp'] or (ntimes < self['numexp'] and 
+                                            not tms_missing)):
+                self['numexp'] = ntimes
                 
+            self['numsub'] = []
+            self['date_obs'] = []
+            self['jd'] = []
+            self['year'] = []
+            self['date_avg'] = []
+            self['jd_avg'] = []
+            self['year_avg'] = []
+            self['date_weighted'] = []
+            self['jd_weighted'] = []
+            self['year_weighted'] = []
+            self['date_end'] = []
+            self['jd_end'] = []
+            self['year_end'] = []
+            self.exposures = []
+            exptime_calc = []
+
             for iexp in np.arange(ntimes):
-                if len(self['date_orig']) > 1:
+                if (isinstance(self['date_orig'], list) and 
+                    len(self['date_orig']) > 1):
                     date_orig = self['date_orig'][iexp]
-                else:
+                elif isinstance(self['date_orig'], list):
                     date_orig = self['date_orig'][0]
+                else:
+                    date_orig = self['date_orig']
+
+                try:
+                    date_test = Time(date_orig, scale='tai')
+                except ValueError:
+                    # Cannot work without valid date_orig
+                    continue
 
                 if self['tms_orig']:
                     tms_orig = self['tms_orig'][iexp]
@@ -1439,114 +1524,241 @@ class PlateMeta(OrderedDict):
                 else:
                     tme_orig = None
 
-                ut_start_isot = None
-                ut_end_isot = None
+                if tms_orig and '|' in tms_orig:
+                    tms_orig = [x.strip() for x in tms_orig.split('|')]
 
-                if ((self['tz_orig'] == 'ST') and (tms_orig or tme_orig) and 
-                    self['site_latitude'] and self['site_longitude']):
-                    # Convert sidereal time to UT using pyEphem's next_transit
-                    # Initialize location and date
-                    loc = ephem.Observer()
-                    loc.lat = str(self['site_latitude'])
-                    loc.lon = str(self['site_longitude'])
+                if tme_orig and '|' in tme_orig:
+                    tme_orig = [x.strip() for x in tme_orig.split('|')]
 
-                    if self['site_elevation']:
-                        loc.elevation = self['site_elevation']
-                        
-                    loc.date = '%s %s' % (date_orig, '12:00:00')
-                    # Define imaginary star with the RA value of the ST of observation,
-                    # and the Dec value of 0. Then compute transit time.
-                    st = ephem.FixedBody()
-                    st._dec = 0.
-                    st._epoch = date_orig
+                try:
+                    exptime = self['exptime'][iexp]
+                except Exception:
+                    exptime = None
+                            
+                if (isinstance(tms_orig, list) or isinstance(tme_orig, list)):
+                    expmeta = self.copy()
+                    expmeta['tms_orig'] = tms_orig
+                    expmeta['tme_orig'] = tme_orig
+                    expmeta['numexp'] = len(tms_orig)
+                    expmeta['exptime'] = []
+                    expmeta.compute_values()
+                    self.exposures.append(expmeta)
 
-                    if tms_orig:
-                        st._ra = ephem.hours(tms_orig)
-                        st.compute()
-                        ut_start_isot = '%04d-%02d-%02dT%02d:%02d:%02d' % loc.next_transit(st).tuple()
+                    # Copy sub-exposure values
+                    self['numsub'].append(expmeta['numexp'])
+                    self['date_obs'].append(expmeta['date_obs'][0])
+                    self['jd'].append(expmeta['jd'][0])
+                    self['year'].append(expmeta['year'][0])
+                    self['date_end'].append(expmeta['date_end'][-1])
+                    self['jd_end'].append(expmeta['jd_end'][-1])
+                    self['year_end'].append(expmeta['year_end'][-1])
 
-                    if tme_orig:
-                        st._ra = ephem.hours(tme_orig)
-                        st.compute()
-                        ut_end_isot = '%04d-%02d-%02dT%02d:%02d:%02d' % loc.next_transit(st).tuple()
-
-                elif self['tz_orig'] == 'UT':
-                    if self['ut_start_orig']:
-                        ut_start_orig = self['ut_start_orig'][iexp]
-
-                        if not ':' in ut_start_orig:
-                            ut_start_orig += ':00'
-
-                        ut_start_isot = '{}T{}'.format(date_orig, ut_start_orig)
-
-                        # Make sure that ut_start_isot formatting is correct
-                        if not '.' in ut_start_orig:
-                            ut_start_isot = Time(ut_start_isot, format='isot', 
-                                                 scale='ut1', precision=0).isot
-
-                    if self['ut_end_orig']:
-                        ut_end_orig = self['ut_end_orig'][iexp]
-
-                        if not ':' in ut_end_orig:
-                            ut_start_orig += ':00'
-
-                        ut_end_isot = '{}T{}'.format(date_orig, ut_end_orig)
-
-                        # Make sure that ut_start_isot formatting is correct
-                        if not '.' in ut_end_orig:
-                            ut_end_isot = Time(ut_end_isot, format='isot',
-                                               scale='ut1', precision=0).isot
-
-                if not tms_orig:
-                    ut_start_isot = Time(date_orig, format='iso', scale='ut1', 
-                                         out_subfmt='date').iso
-
-                if ut_start_isot:
-                    self['date_obs'].append(ut_start_isot)
-                    time_start = Time(ut_start_isot, format='isot', scale='ut1')
-
-                    if tms_orig:
-                        self['jd'].append(float('%.5f' % time_start.jd))
-                        self['year'].append(float('%.8f' % time_start.jyear))
+                    # Check if exptimes exist for all sub-exposures
+                    if (len(filter(None, expmeta['exptime'])) == 
+                        expmeta['numexp']):
+                        exptime_calc.append(sum(expmeta['exptime']))
+                        jd_weighted = np.average(expmeta['jd_avg'],
+                                                 weights=expmeta['exptime'])
+                        year_weighted = np.average(expmeta['year_avg'],
+                                                   weights=expmeta['exptime'])
+                        time_weighted = Time(jd_weighted, format='jd', 
+                                             scale='ut1', precision=0)
+                        self['date_weighted'].append(time_weighted.isot)
+                        self['jd_weighted'].append(jd_weighted)
+                        self['year_weighted'].append(year_weighted)
                     else:
-                        self['jd'].append(float('%.0f' % time_start.jd) + 1.)
-                        self['year'].append(float('%.3f' % time_start.jyear))
+                        exptime_calc.append(None)
+                        self['date_weighted'].append(None)
+                        self['jd_weighted'].append(None)
+                        self['year_weighted'].append(None)
 
-                    try:
-                        exptime = self['exptime'][iexp]
-                    except:
-                        exptime = None
-                        
-                    if exptime and tms_orig:
-                        time_avg = Time(time_start.jd + 0.5 * exptime / 86400., format='jd', 
-                            scale='ut1', precision=0)
-                        self['date_avg'].append(time_avg.isot)
-                        self['jd_avg'].append(float('%.5f' % time_avg.jd))
-                        self['year_avg'].append(float('%.8f' % time_avg.jyear))
-                    else:
-                        self['date_avg'].append(None)
-                        self['jd_avg'].append(None)
-                        self['year_avg'].append(None)
+                    jd_avg = np.mean([expmeta['jd'][0], expmeta['jd_end'][-1]])
+                    year_avg = np.mean([expmeta['year'][0],
+                                        expmeta['year_end'][-1]])
+                    time_avg = Time(jd_avg, format='jd', scale='ut1', 
+                                    precision=0)
+                    self['date_avg'].append(time_avg.isot)
+                    self['jd_avg'].append(float('{:.5f}'.format(jd_avg)))
+                    self['year_avg'].append(float('{:.8f}'.format(year_avg)))
                 else:
-                    self['date_obs'].append(None)
-                    self['jd'].append(None)
+                    self['numsub'].append(1)
+                    self.exposures.append(None)
 
-                if ut_end_isot:
-                    self['date_end'].append(ut_end_isot)
-                    time_end = Time(ut_end_isot, format='isot', scale='ut1')
+                    ut_start_isot = None
+                    ut_end_isot = None
 
-                    if tme_orig:
-                        self['jd_end'].append(float('{:.5f}'
-                                                    .format(time_end.jd)))
-                        self['year_end'].append(float('{:.8f}'
-                                                      .format(time_end.jyear)))
+                    # Handle cases where time hours are larger than 24
+                    if tms_orig or tme_orig:
+                        t_date_orig = Time(date_orig, scale='tai')
+
+                        if tme_orig:
+                            if tme_orig.count(':') == 1:
+                                tme_orig += ':00'
+
+                            tsec = pytimeparse.parse(tme_orig)
+
+                            if tsec > 86400:
+                                td_tme = TimeDelta(tsec, format='sec')
+                                t_tme = t_date_orig + td_tme
+                                date_orig = Time(t_tme, out_subfmt='date').iso
+                                tme_orig = Time(t_tme).iso.split()[-1]
+
+                        if tms_orig:
+                            if tms_orig.count(':') == 1:
+                                tms_orig += ':00'
+
+                            tsec = pytimeparse.parse(tms_orig)
+                            
+                            if tsec > 86400:
+                                td_tms = TimeDelta(tsec, format='sec')
+                                t_tms = t_date_orig + td_tms
+                                date_orig = Time(t_tms, out_subfmt='date').iso
+                                tms_orig = Time(t_tms).iso.split()[-1]
+
+                    if ((self['tz_orig'] == 'ST') and (tms_orig or tme_orig) and 
+                        self['site_latitude'] and self['site_longitude']):
+                        # Convert sidereal time to UT using pyEphem's next_transit
+                        # Initialize location and date
+                        loc = ephem.Observer()
+                        loc.lat = str(self['site_latitude'])
+                        loc.lon = str(self['site_longitude'])
+
+                        if self['site_elevation']:
+                            loc.elevation = self['site_elevation']
+                            
+                        loc.date = '%s %s' % (date_orig, '12:00:00')
+                        # Define imaginary star with the RA value of the ST of observation,
+                        # and the Dec value of 0. Then compute transit time.
+                        st = ephem.FixedBody()
+                        st._dec = 0.
+                        st._epoch = date_orig
+
+                        if tms_orig:
+                            st._ra = ephem.hours(tms_orig)
+                            st.compute()
+                            ut_start_isot = '%04d-%02d-%02dT%02d:%02d:%02d' % loc.next_transit(st).tuple()
+
+                        if tme_orig:
+                            st._ra = ephem.hours(tme_orig)
+                            st.compute()
+                            ut_end_isot = '%04d-%02d-%02dT%02d:%02d:%02d' % loc.next_transit(st).tuple()
+
+                    elif self['tz_orig'] == 'UT':
+                        if self['ut_start_orig']:
+                            ut_start_orig = self['ut_start_orig'][iexp]
+
+                            if not ':' in ut_start_orig:
+                                ut_start_orig += ':00'
+
+                            ut_start_isot = '{}T{}'.format(date_orig, ut_start_orig)
+
+                            # Make sure that ut_start_isot formatting is correct
+                            if not '.' in ut_start_orig:
+                                ut_start_isot = Time(ut_start_isot, format='isot', 
+                                                     scale='ut1', precision=0).isot
+
+                        if self['ut_end_orig']:
+                            ut_end_orig = self['ut_end_orig'][iexp]
+
+                            if not ':' in ut_end_orig:
+                                ut_end_orig += ':00'
+
+                            ut_end_isot = '{}T{}'.format(date_orig, ut_end_orig)
+
+                            # Check if end time is after midnight. If so, use next
+                            # date.
+                            if (self['ut_start_orig'] and 
+                                ut_end_orig < ut_start_orig):
+                                next_day = (Time(ut_end_isot, format='isot', 
+                                                 scale='ut1', precision=0) 
+                                            + TimeDelta(1, format='jd'))
+                                date_end = next_day.isot.split('T')[0]
+                                ut_end_isot = '{}T{}'.format(date_end, ut_end_orig)
+
+                            # Make sure that ut_start_isot formatting is correct
+                            if not '.' in ut_end_orig:
+                                ut_end_isot = Time(ut_end_isot, format='isot',
+                                                   scale='ut1', precision=0).isot
+
+                    if not tms_orig:
+                        ut_start_isot = Time(date_orig, format='iso', scale='ut1', 
+                                             out_subfmt='date').iso
+
+                    if ut_start_isot:
+                        self['date_obs'].append(ut_start_isot)
+                        time_start = Time(ut_start_isot, format='isot', scale='ut1')
+
+                        if tms_orig:
+                            self['jd'].append(float('%.5f' % time_start.jd))
+                            self['year'].append(float('%.8f' % time_start.jyear))
+                        else:
+                            self['jd'].append(float('%.0f' % time_start.jd) + 1.)
+                            self['year'].append(float('%.3f' % time_start.jyear))
                     else:
+                        self['date_obs'].append(None)
+                        self['jd'].append(None)
+                        self['year'].append(None)
+
+                    if ut_end_isot:
+                        self['date_end'].append(ut_end_isot)
+                        time_end = Time(ut_end_isot, format='isot', scale='ut1')
+
+                        if tme_orig:
+                            self['jd_end'].append(float('{:.5f}'
+                                                        .format(time_end.jd)))
+                            self['year_end'].append(float('{:.8f}'
+                                                          .format(time_end.jyear)))
+                        else:
+                            self['jd_end'].append(None)
+                            self['year_end'].append(None)
+                    else:
+                        self['date_end'].append(None)
                         self['jd_end'].append(None)
                         self['year_end'].append(None)
-                else:
-                    self['date_end'].append(None)
-                    self['jd_end'].append(None)
-                    self['year_end'].append(None)
+
+                    if ut_start_isot and ut_end_isot:
+                        jd_avg = np.mean([time_start.jd, time_end.jd])
+                        year_avg = np.mean([time_start.jyear, time_end.jyear])
+                        time_avg = Time(jd_avg, format='jd', scale='ut1', 
+                                        precision=0)
+                        self['date_avg'].append(time_avg.isot)
+                        self['date_weighted'].append(time_avg.isot)
+                        jd_avg = float('{:.5f}'.format(jd_avg))
+                        self['jd_avg'].append(jd_avg)
+                        self['jd_weighted'].append(jd_avg)
+                        year_avg = float('{:.8f}'.format(year_avg))
+                        self['year_avg'].append(year_avg)
+                        self['year_weighted'].append(year_avg)
+                    elif ut_start_isot and exptime:
+                        time_avg = Time(time_start.jd + 0.5 * exptime / 86400.,
+                                        format='jd', scale='ut1', precision=0)
+                        jd_avg = float('{:.5f}'.format(time_avg.jd))
+                        year_avg = float('{:.8f}'.format(time_avg.jyear))
+                        self['date_avg'].append(time_avg.isot)
+                        self['date_weighted'].append(time_avg.isot)
+                        self['jd_avg'].append(jd_avg)
+                        self['jd_weighted'].append(jd_avg)
+                        self['year_avg'].append(year_avg)
+                        self['year_weighted'].append(year_avg)
+                    else:
+                        self['date_avg'].append(None)
+                        self['date_weighted'].append(None)
+                        self['jd_avg'].append(None)
+                        self['jd_weighted'].append(None)
+                        self['year_avg'].append(None)
+                        self['year_weighted'].append(None)
+
+                    if ut_start_isot and ut_end_isot:
+                        tdiff = (Time(ut_end_isot, format='isot', 
+                                      scale='tai') -
+                                 Time(ut_start_isot, format='isot', 
+                                      scale='tai'))
+                        exptime_calc.append(tdiff.sec)
+                    else:
+                        exptime_calc.append(None)
+
+            if self['exptime'] == [] and filter(None, exptime_calc) != []:
+                self['exptime'] = exptime_calc
 
         if self['ra_orig'] and self['dec_orig'] and self['date_orig']:
             for iexp in np.arange(len(self['ra_orig'])):
@@ -1563,7 +1775,7 @@ class PlateMeta(OrderedDict):
 
                 try:
                     ra_str = Angle(pointing.ra, units.radian)\
-                            .to_string(unit=units.deg, sep=':', precision=1, 
+                            .to_string(unit=units.hour, sep=':', precision=1, 
                                        pad=True)
                     dec_str = Angle(pointing.dec, units.radian)\
                             .to_string(unit=units.deg, sep=':', precision=1, 
@@ -1943,6 +2155,11 @@ class PlateHeader(fits.Header):
         """
 
         if value or (valtype is int and value == 0):
+            if isinstance(value, str):
+                value = (value.replace('ä','ae').replace('Ä','AE')
+                         .replace('ö','oe').replace('Ö','OE')
+                         .replace('ü','ue').replace('Ü','UE'))
+
             self.set(key, value)
         elif not key in self:
             if valtype is str:
@@ -2219,6 +2436,8 @@ class PlateHeader(fits.Header):
             a = Angle(self['DEC'], units.degree)
             self.set('DEC_DEG', float('%.4f' % a.degrees))
 
+        self.set('DATE', dt.datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%S'))
+
     def update_comments(self):
         """
         Add/modify keyword comments based on configuration.
@@ -2404,6 +2623,8 @@ class PlateHeader(fits.Header):
             self.add_history('WCS added with PyPlate v{} at {}'
                              .format(__version__, dt.datetime.utcnow()
                                      .strftime('%Y-%m-%dT%H:%M:%S')))
+            self.set('DATE', 
+                     dt.datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%S'))
 
     def output_to_fits(self, filename):
         """
@@ -2416,14 +2637,16 @@ class PlateHeader(fits.Header):
 
         if os.path.exists(fn_out):
             fitsfile = fits.open(fn_out, mode='update', 
-                                 do_not_scale_image_data=True)
+                                 do_not_scale_image_data=True, 
+                                 ignore_missing_end=True)
             fitsfile[0].header = self.copy()
             fitsfile.flush()
         else:
             if not os.path.exists(fn_fits):
                 print 'File does not exist: {}'.format(fn_fits)
 
-            fitsfile = fits.open(fn_fits, do_not_scale_image_data=True)
+            fitsfile = fits.open(fn_fits, do_not_scale_image_data=True, 
+                                 ignore_missing_end=True)
             fitsfile[0].header = self.copy()
 
             try:
