@@ -1,23 +1,27 @@
 import os
-from metadata import ArchiveMeta, PlateMeta, PlateHeader, read_conf
-from solve import SolveProcess
-from database import PlateDB
 import multiprocessing as mp
 import time
+import ConfigParser
+from .metadata import ArchiveMeta, PlateMeta, PlateHeader, read_conf
+from .solve import SolveProcess
+from .database import PlateDB
+from .image import PlateConverter
 
 
-class PlateImagePipeline():
+class PlateImagePipeline:
     """
     Plate processing pipeline class
 
     """
 
-    def __init__(self):
+    def __init__(self, plate_converter=None):
         self.conf = None
         self.work_dir = ''
         self.write_log_dir = ''
         self.input_queue = None
         self.done_queue = None
+        self.plate_converter = plate_converter
+        self.plate_epoch = None
 
         self.read_wfpdb = False
         self.read_csv = False
@@ -27,8 +31,11 @@ class PlateImagePipeline():
         self.invert_image = False
         self.extract_sources = False
         self.solve_plate = False
+        self.output_solution_db = False
         self.output_wcs_file = False
         self.solve_recursive = False
+        self.calibrate_photometry = False
+        self.output_calibration_db = False
         self.output_sources_db = False
         self.output_sources_csv = False
 
@@ -52,7 +59,9 @@ class PlateImagePipeline():
         for attr in ['read_wfpdb', 'read_csv', 'read_fits', 
                      'output_header_file', 'output_header_fits', 
                      'invert_image', 'extract_sources', 'solve_plate', 
-                     'output_wcs_file', 'solve_recursive', 
+                     'output_solution_db', 'output_wcs_file', 
+                     'solve_recursive', 'calibrate_photometry', 
+                     'output_calibration_db', 
                      'output_sources_db', 'output_sources_csv']:
             try:
                 setattr(self, attr, conf.getboolean('Pipeline', attr))
@@ -62,7 +71,7 @@ class PlateImagePipeline():
             except ConfigParser.Error:
                 pass
 
-    def single_image(self, filename):
+    def single_image(self, filename, plate_epoch=None):
         """
         Process single plate image.
 
@@ -70,6 +79,8 @@ class PlateImagePipeline():
         ----------
         filename : str
             Filename of the FITS image to be processed
+        plate_epoch : float
+            Plate epoch (decimal year)
             
         """
 
@@ -109,6 +120,13 @@ class PlateImagePipeline():
         proc = SolveProcess(fn)
         proc.assign_conf(pmeta.conf)
         proc.assign_header(h)
+
+        if self.plate_epoch is not None:
+            proc.plate_epoch = self.plate_epoch
+
+        if plate_epoch is not None:
+            proc.plate_epoch = plate_epoch
+
         proc.setup()
 
         if self.invert_image:
@@ -120,14 +138,41 @@ class PlateImagePipeline():
             if self.solve_plate:
                 proc.solve_plate()
 
+            if self.output_solution_db:
+                proc.output_solution_db()
+
             if self.output_wcs_file:
                 proc.output_wcs_header()
 
+            if proc.solution is not None:
+                proc.log.write('Updating FITS header with the WCS', 
+                               level=3, event=47)
+                h.insert_wcs(proc.solution['wcs'])
+
+            if self.output_header_file:
+                proc.log.write('Writing FITS header to a file', 
+                               level=3, event=48)
+                h.output_to_file(fn_header)
+
+            if self.output_header_fits:
+                proc.log.write('Writing FITS header to the FITS file', 
+                               level=3, event=49)
+                h.output_to_fits(fn)
+
             if self.solve_recursive:
                 proc.solve_recursive()
 
             proc.process_source_coordinates()
 
+            if self.calibrate_photometry:
+                proc.calibrate_photometry()
+
+            if self.output_calibration_db:
+                proc.output_cterm_db()
+                proc.output_color_db()
+                proc.output_calibration_db()
+                proc.output_rmse_db()
+
             if self.output_sources_db:
                 proc.output_sources_db()
 
@@ -154,7 +199,13 @@ class PlateImagePipeline():
             if fn == 'DONE':
                 break
 
-            self.single_image(fn)
+            if self.plate_converter:
+                plateconv = PlateConverter()
+                plateconv.assign_conf(self.conf)
+                plateconv.tiff2fits(fn)
+            else:
+                self.single_image(fn)
+
             self.done_queue.put(fn)
 
     def parallel_run(self, filenames, processes=1):
@@ -193,6 +244,8 @@ class PlateImagePipeline():
             job = mp.Process(target=self.worker)
             job.start()
             jobs.append(job)
+            # Wait 10 seconds before starting another process
+            time.sleep(10)
 
         # Write unfinished and finished file lists to disk every 10 seconds
         while True:
